@@ -138,6 +138,7 @@ skip_platform() {
 # note: we depend on CFLAGS to properly reflect build options (what
 #       usually is the case when the testbench is run)
 skip_TSAN() {
+	echo skip:_TSAN, CFLAGS $CFLAGS
 	if [[ "$CFLAGS" == *"sanitize=thread"* ]]; then
 		printf 'test incompatible with TSAN because of %s\n' "$1"
 		exit 77
@@ -983,7 +984,7 @@ wait_queueempty() {
 
 # shut rsyslogd down when main queue is empty. $1 is the instance.
 shutdown_when_empty() {
-	echo Shutting down instance ${1:-1}
+	printf '%s Shutting down instance %s\n' "$(date +%H:%M:%S)" ${1:-1}
 	wait_queueempty $1
 	if [ "$RSYSLOG_PIDBASE" == "" ]; then
 		echo "RSYSLOG_PIDBASE is EMPTY! - bug in test? (instance $1)"
@@ -2280,6 +2281,7 @@ ensure_elasticsearch_ready() {
 	then
 		printf 'Elasticsearch already running, NOT restarting it\n'
 	else
+		printf '%s Elasticsearch not yet running, starting it\n' "$(date +%H:%M:%S)"
 		cat elasticsearch.running
 		cleanup_elasticsearch
 		dep_es_cached_file="$dep_cache_dir/$ES_DOWNLOAD"
@@ -2294,6 +2296,7 @@ ensure_elasticsearch_ready() {
 		printf 'running elasticsearch instance: %s\n' "$(cat elasticsearch.running)"
 		init_elasticsearch
 	fi
+	printf '\n%s Elasticsearch readied for use as requested\n' "$(date +%H:%M:%S)"
 }
 
 
@@ -2318,6 +2321,7 @@ start_elasticsearch() {
 	timeseconds=0
 	# Loop until elasticsearch port is reachable or until
 	# timeout is reached!
+	curl --silent --show-error --connect-timeout 1 http://localhost:${ES_PORT:-19200}
 	until [ "$(curl --silent --show-error --connect-timeout 1 http://localhost:${ES_PORT:-19200} | grep 'rsyslog-testbench')" != "" ]; do
 		echo "--- waiting for ES startup: $timeseconds seconds"
 		$TESTTOOL_DIR/msleep 1000
@@ -2346,10 +2350,12 @@ start_elasticsearch() {
 # $1 - number of records (ES does not return all records unless you tell it explicitly).
 # $2 - ES port
 es_getdata() {
+	printf '%s getting data from ElasticSearch\n' "$(date +%H:%M:%S)"
 	curl --silent -XPUT --show-error -H 'Content-Type: application/json' "http://localhost:${2:-$ES_PORT}/rsyslog_testbench/_settings" -d '{ "index" : { "max_result_window" : '${1:-$NUMMESSAGES}' } }'
 	# refresh to ensure we get the latest data
 	curl --silent localhost:${2:-$ES_PORT}/rsyslog_testbench/_refresh
 	curl --silent localhost:${2:-$ES_PORT}/rsyslog_testbench/_search?size=${1:-$NUMMESSAGES} > $RSYSLOG_DYNNAME.work
+	printf '\n'
 	$PYTHON $srcdir/es_response_get_msgnum.py > ${RSYSLOG_OUT_LOG}
 }
 
@@ -2371,7 +2377,7 @@ es_shutdown_empty_check() {
 stop_elasticsearch() {
 	dep_work_dir=$(readlink -f $srcdir)
 	dep_work_es_pidfile="es.pid"
-	rm elasticsearch.running
+	rm -f elasticsearch.running
 	if [ -e $dep_work_es_pidfile ]; then
 		es_pid=$(cat $dep_work_es_pidfile)
 		printf 'stopping ES with pid %d\n' $es_pid
@@ -2651,9 +2657,17 @@ first_column_sum_check() {
 # Helper functions to start/stop python snmp trap receiver
 #
 snmp_start_trapreceiver() {
+	SNMP_PYTHON=${SNMP_PYTHON:-$PYTHON}
     # Args: 1=port 2=outputfilename
     # Args 2 and up are passed along as is to snmptrapreceiver.py
-    snmptrapreceiver=$srcdir/snmptrapreceiver.py
+    echo "Checking Python version..."
+    if $SNMP_PYTHON -c 'import sys; print("Python version:", sys.version_info); exit(0 if sys.version_info >= (3,11) else 1)' 2>/dev/null; then
+        echo "Python version > 3.10, using snmptrapreceiverv2.py"
+        snmptrapreceiver=$srcdir/snmptrapreceiverv2.py
+    else
+        echo "Python version < 3.10, using snmptrapreceiver.py"
+        snmptrapreceiver=$srcdir/snmptrapreceiver.py
+    fi
     if [ ! -f ${snmptrapreceiver} ]; then
         echo "Cannot find ${snmptrapreceiver} for omsnmp test"
         error_exit 1
@@ -2679,31 +2693,45 @@ snmp_start_trapreceiver() {
     mkdir -p ${snmp_work_dir}
 
     server_args="${snmp_server_port} 127.0.0.1 ${output_file}"
-
-    $PYTHON ${snmptrapreceiver} ${server_args} ${snmp_server_logfile} >> ${snmp_server_logfile} 2>&1 &
-    if [ ! $? -eq 0 ]; then
-        echo "Failed to start snmptrapreceiver."
-        rm -rf ${snmp_work_dir}
-        error_exit 1
-    fi
-
+    echo "RUN SNMP SERVER WITH: $SNMP_PYTHON ${snmptrapreceiver} ${server_args} ${snmp_server_logfile} >> ${snmp_server_logfile} 2>&1"
+    $SNMP_PYTHON ${snmptrapreceiver} ${server_args} ${snmp_server_logfile} >> ${snmp_server_logfile} 2>&1 &
     snmp_server_pid=$!
+    if ! ps -p $snmp_server_pid > /dev/null 2>&1; then
+        echo "Failed to start snmptrapreceiver wiht $SNMP_PYTHON."
+        if [ "$SNMP_PYTHON" = "/usr/bin/python" ]; then
+            SNMP_PYTHON="/usr/bin/python3"
+			rm -rf ${snmp_work_dir}
+		    mkdir -p ${snmp_work_dir}
+            echo "Retrying with python3..."
+		    echo "RUN SNMP SERVER WITH: $SNMP_PYTHON ${snmptrapreceiver} ${server_args} ${snmp_server_logfile} >> ${snmp_server_logfile} 2>&1"
+            $SNMP_PYTHON ${snmptrapreceiver} ${server_args} ${snmp_server_logfile} >> ${snmp_server_logfile} 2>&1 &
+			snmp_server_pid=$!
+			if ! ps -p $snmp_server_pid > /dev/null 2>&1; then
+                echo "Failed to start snmptrapreceiver with $SNMP_PYTHON."
+				rm -rf ${snmp_work_dir}
+                error_exit 1
+            fi
+        fi
+    fi
     echo ${snmp_server_pid} > ${snmp_server_pidfile}
 
+	while test ! -f ${snmp_server_logfile}.started; do
+		$TESTTOOL_DIR/msleep 100 # wait 100 milliseconds
+		printf "."
+	done
+
     while test ! -s "${snmp_server_logfile}"; do
-	$TESTTOOL_DIR/msleep 100 # wait 100 milliseconds
-	if [ $(date +%s) -gt $(( TB_STARTTEST + TB_TEST_MAX_RUNTIME )) ]; then
-	   printf '%s ABORT! Timeout waiting on startup (pid file %s)\n' "$(tb_timestamp)" "$1"
-	   ls -l "$1"
-	   ps -fp $(cat "$1")
-	   snmp_stop_trapreceiver
-	   error_exit 1
-#	else 
-#	   echo "waiting...${snmp_server_logfile}..."
-	fi
+		$TESTTOOL_DIR/msleep 100 # wait 100 milliseconds
+		if [ $(date +%s) -gt $(( TB_STARTTEST + TB_TEST_MAX_RUNTIME )) ]; then
+		printf '%s ABORT! Timeout waiting on startup (pid file %s)\n' "$(tb_timestamp)" "$1"
+		ls -l "$1"
+		ps -fp $(cat "$1")
+		snmp_stop_trapreceiver
+		error_exit 1
+		fi
     done
 
-    echo "Started snmptrapreceiver with args ${server_args} with pid ${snmp_server_pid}"
+    echo "Started snmptrapreceiver with ${SNMP_PYTHON} args ${server_args} with pid ${snmp_server_pid}"
 }
 
 snmp_stop_trapreceiver() {
@@ -2715,6 +2743,8 @@ snmp_stop_trapreceiver() {
         echo "Stopping snmptrapreceiver server"
         kill -9 $(cat ${snmp_work_dir}/snmp_server.pid) > /dev/null 2>&1
         # Done at testexit already!: rm -rf ${snmp_work_dir}
+	echo "SNMP Trap Receiver log:"
+	cat ${snmp_work_dir}//snmp_server.log
     fi
 }
 
