@@ -1,12 +1,12 @@
 #!/bin/bash
 # addd 2017-03-06 by RGerhards, released under ASL 2.0
 
-# Note: we need to inject a somewhat larger nubmer of messages in order
+# Note: we need to inject a somewhat larger number of messages in order
 # to ensure that we receive some messages in the actual output file,
 # as batching can (validly) cause a larger loss in the non-writable
 # file
 
-. $srcdir/diag.sh init
+. ${srcdir:=.}/diag.sh init
 generate_conf
 add_conf '
 global(umask="0077")
